"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// packages/core/src/index.ts
var src_exports = {};
__export(src_exports, {
  useUrlState: () => useUrlState,
  useUrlStateWithRouter: () => useUrlStateWithRouter
});
module.exports = __toCommonJS(src_exports);
var import_react2 = require("react");

// packages/core/src/handlers.ts
var import_react = require("react");
function usePush(router) {
  return (0, import_react.useCallback)(
    (href) => {
      router.push(href);
    },
    [router]
  );
}
function useHandlers(push, stateRef) {
  const reset = (0, import_react.useCallback)(() => {
    const href = `?${new URLSearchParams({}).toString()}`;
    push(href);
  }, [push]);
  const replace = (0, import_react.useCallback)(
    (data) => {
      const href = `?${new URLSearchParams(data).toString()}`;
      push(href);
    },
    [push]
  );
  const setValue = (0, import_react.useCallback)(
    (key, value) => {
      const href = `?${new URLSearchParams({
        ...stateRef.current.data,
        [key]: value
      }).toString()}`;
      push(href);
    },
    [push, stateRef]
  );
  return { reset, replace, setValue };
}

// packages/core/src/router.ts
var UrlStateRouter = class {
  push(href) {
  }
  onSearchParamsChange() {
  }
  destroy() {
  }
};
var genericRouterCurrentStateString = "";
var GenericRouter = class extends UrlStateRouter {
  constructor(callback) {
    super();
    this.callback = callback;
    this.interval = 0;
    if (typeof window !== "undefined") {
      this.interval = setInterval(() => {
        this.onSearchParamsChange();
      }, 100);
    }
  }
  push(href) {
    window.history.pushState({}, "", href);
    this.onSearchParamsChange();
  }
  onSearchParamsChange() {
    if (window.location.search !== genericRouterCurrentStateString) {
      genericRouterCurrentStateString = window.location.search;
      this.callback(window.location.search);
    }
  }
  destroy() {
    if (this.interval) {
      clearInterval(this.interval);
    }
  }
};

// packages/core/src/utils.ts
function urlParamsToObject(params) {
  const object = {};
  for (const [key, value] of params.entries()) {
    if (!object[key]) {
      object[key] = value;
      continue;
    }
    const currentValue = object[key];
    if (typeof currentValue === "string") {
      object[key] = [currentValue, value];
    } else {
      currentValue.push(value);
    }
  }
  return object;
}

// packages/core/src/index.ts
var subscribers = /* @__PURE__ */ new Map();
var currentStateString = "";
function update(newSearchParams) {
  console.log("Update");
  if (newSearchParams !== currentStateString) {
    currentStateString = newSearchParams;
    subscribers.forEach((subscriber) => subscriber(currentStateString));
  }
}
function subscribeToHistory(fn) {
  subscribers.set(fn, fn);
}
function unsubscribeFromHistory(fn) {
  subscribers.delete(fn);
}
function useUrlState(schema, options) {
  const [router] = (0, import_react2.useState)(new GenericRouter(update));
  (0, import_react2.useEffect)(() => {
    return () => {
      router.destroy();
    };
  }, [router]);
  return useUrlStateWithRouter(schema, {
    ...options,
    preserveUnknown: false,
    router
  });
}
function useUrlStateWithRouter(schema, options) {
  const schemaRef = (0, import_react2.useRef)(schema);
  const [state, setState] = (0, import_react2.useState)({
    data: null,
    isError: false,
    error: null,
    isReady: false
  });
  const stateRef = (0, import_react2.useRef)(state);
  stateRef.current = state;
  const recalculateState = (0, import_react2.useCallback)(
    (searchString) => {
      const params = new URLSearchParams(searchString);
      const object = urlParamsToObject(params);
      const validationResult = schemaRef.current.safeParse(object);
      const result = validationResult.success ? { success: true, data: validationResult.data, error: null } : { success: false, data: null, error: validationResult.error };
      if (options?.preserveUnknown) {
        result.data = { ...object, ...result.data };
      }
      setState({
        data: result.data,
        isError: !result.success,
        error: result.error,
        isReady: true
      });
    },
    [options?.preserveUnknown]
  );
  (0, import_react2.useEffect)(() => {
    subscribeToHistory(recalculateState);
    return () => {
      unsubscribeFromHistory(recalculateState);
    };
  }, [recalculateState]);
  (0, import_react2.useEffect)(() => {
    update(window.location.search);
  }, []);
  const push = usePush(options.router);
  const handlers = useHandlers(push, stateRef);
  return { ...state, ...handlers };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useUrlState,
  useUrlStateWithRouter
});
